// Code based on the ESP32 example code TouchInterupt and code from 
// https://iotdesignpro.com/projects/creating-nodemcu-mesh-networking-using-esp12-and-arduino-ide
//Touchpin numberbering can be found at https://gitlab.fabcloud.org/barcelonaworkshops/barduino-2.0/ 
// Program runs painlessMesh to create a mesh network
// Operates 5 touchpins. When pin is touched a message is send to both the serial monitor and the oled monitor
// When a message is received it outputs to both serial and oled monitor.
// position of cursor is communicated to the monitor. 
// When cursor is at the end of the monitor, display.cleardisplay is triggered.  



// Code for Node1:
#include "painlessMesh.h"   // include painlessMesh library
#include <ArduinoJson.h>
#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>


// WiFi Credentials
#define   MESH_PREFIX     "Post-collapseNetwork"
#define   MESH_PASSWORD   "TechnoLogic"
#define   MESH_PORT       5555

#define SCREEN_WIDTH 128 // OLED display width, in pixels
#define SCREEN_HEIGHT 64 // OLED display height, in pixels

// Necessary for OLED Display
#define OLED_RESET     -1 // Reset pin # (or -1 if sharing Arduino reset pin)
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);

int threshold = 17; // setting threshold for detecting touch on pin
// int threshold2 = 99; 
bool touch1detected = false;   
bool touch2detected = false;
bool touch3detected = false;
bool touch4detected = false;
bool touch5detected = false;
bool touch6detected = false;

//Pin Declaration
#define touchPin1 T2  // declaring touch pins
#define touchPin2 T6 
#define touchPin3 T7 
#define touchPin4 T8
#define touchPin5 T3
#define touchPin6 T5

Scheduler userScheduler; // calling on the scheduler to control your personal task. This replaces 'delay' function which messes up painlessMesh 
painlessMesh  mesh;  // calling on painlessMesh

void sendMessage() ;  // Prototype so PlatformIO doesn't complain
Task taskSendMessage( TASK_SECOND * 1 , TASK_FOREVER, &sendMessage );  // this command replaces the 'delay' function
void sendMessage() 
{
// Reading Status of touchPin
  if (digitalRead(touchPin1) == HIGH)
    touch1detected = !touch1detected;  // creates container to put status of pin in.
  if (digitalRead(touchPin2) == HIGH)
    touch2detected = !touch2detected;
  if (digitalRead(touchPin3) == HIGH)
    touch3detected = !touch3detected;
  if (digitalRead(touchPin4) == HIGH)
    touch4detected = !touch4detected;
  if (digitalRead(touchPin5) == HIGH)
    touch5detected = !touch5detected;
  if (digitalRead(touchPin6) == HIGH)
    touch6detected = !touch6detected;
    
// Serializing in JSON Format 
  DynamicJsonDocument doc(1024);
  doc["Relay1"] = touch1detected; // container created above is made into a doc named Relay1 file.
  doc["Relay2"] = touch2detected; // container created above is made into a doc named Relay2 file.
  doc["Relay3"] = touch3detected; 
  doc["Relay4"] = touch4detected; 
  doc["Relay5"] = touch5detected;
  doc["Relay6"] = touch6detected;
  String msg ;
  serializeJson(doc, msg);   // 
    }


// Needed for painless library
void receivedCallback( uint32_t from, String &msg ) {
  Serial.printf("startHere: Received from %u msg=%s\n", from, msg.c_str());
  checkDisplay();
  display.print(msg.c_str());
  display.display();
}

void newConnectionCallback(uint32_t nodeId) {
  Serial.printf("--> startHere: New Connection, nodeId = %u\n", nodeId);
}
void changedConnectionCallback() {
  Serial.printf("Changed connections\n");
}
void nodeTimeAdjustedCallback(int32_t offset) {
  Serial.printf("Adjusted time %u. Offset = %d\n", mesh.getNodeTime(), offset);
}

void gotTouch1(){
 touch1detected = true;
}
void gotTouch2(){
 touch2detected = true;
}
void gotTouch3(){
 touch3detected = true;
}
void gotTouch4(){
 touch4detected = true;
}
void gotTouch5(){
 touch5detected = true;
}
void gotTouch6(){
 touch6detected = true;
}


void setup() {
  Serial.begin(115200);

  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);// initialize with the I2C addr 0x3C (for the 128x64)
  display.clearDisplay();
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.setCursor(0, 0);
  display.println("post-collapse network");
  display.setTextSize(2);
  display.setCursor(0, 15);
  display.print("Message: ");
  display.setTextColor(WHITE); // 'inverted' text
  display.display();   //you have to tell the display to...display
  
  touchAttachInterrupt(T2, gotTouch1, threshold);  // checks if touchpin is interupted
  touchAttachInterrupt(T6, gotTouch2, threshold);
  touchAttachInterrupt(T7, gotTouch3, threshold);
  touchAttachInterrupt(T8, gotTouch4, threshold);
  touchAttachInterrupt(T3, gotTouch5, threshold);
  touchAttachInterrupt(T5, gotTouch6, threshold);
  
  mesh.setDebugMsgTypes( ERROR | STARTUP );  // set before init() so that you can see startup messages
  mesh.init( MESH_PREFIX, MESH_PASSWORD, &userScheduler, MESH_PORT );
  mesh.onReceive(&receivedCallback);
  mesh.onNewConnection(&newConnectionCallback);
  mesh.onChangedConnections(&changedConnectionCallback);
  mesh.onNodeTimeAdjusted(&nodeTimeAdjustedCallback);
  userScheduler.addTask( taskSendMessage );
  taskSendMessage.enable();
}

void checkDisplay(){
   // int cursorX=display.getCursorX();
   // Serial.println(cursorX);
    int cursorY=display.getCursorY();
    Serial.println(cursorY);
    if(cursorY>53){
      display.clearDisplay(); 
      display.setCursor(0, 0); 
    }
  
}


void loop() {
    mesh.update();
    
  
  if(touch1detected){
    touch1detected = false;
    checkDisplay();
    Serial.println("Touch 1 detected");
    display.print("/");
    display.display();
    
    sendMessage();   // Broadcasting message
    String msg = "/";
    mesh.sendBroadcast( msg );
}
  
  if(touch2detected){
    touch2detected = false;
    checkDisplay();
    Serial.println("Touch 2 detected");
    display.print("-");
    display.display();

  sendMessage();   
    String msg = "-";
    mesh.sendBroadcast( msg );  
  }

 if(touch3detected){
    touch3detected = false;
    checkDisplay();
    Serial.println("Touch 3 detected");
    display.print(" ");
    display.display();

  sendMessage();   
    String msg = " ";
    mesh.sendBroadcast( msg );
 }    

 if(touch4detected){
    touch4detected = false;
    checkDisplay();
    Serial.println("Touch 4 detected");
    display.print(".");
    display.display();

  sendMessage();   
    String msg = ".";
    mesh.sendBroadcast( msg );
 }    
 if(touch5detected){
    touch5detected = false;
    checkDisplay();
    Serial.println("Touch 5 detected");
    display.print("/");
    display.display();

  sendMessage();   
    String msg = "/";
    mesh.sendBroadcast( msg );
 } 
 
 if(touch6detected){
    touch6detected = false;
    checkDisplay();
    Serial.println("Touch 6 detected");
    display.print("/");
    display.display();

  sendMessage();   
    String msg = "/";
    mesh.sendBroadcast( msg );
 }    
}
